#include "echomotion.h"
#include <random>

EchoMotion::EchoMotion()
{
}
EchoMotion::~EchoMotion() {

}

Bool EchoMotion::InitEffector(GeListNode* node) {
	BaseObject* op = (BaseObject*)node;
	if (!op)
		return false;
	BaseContainer* data = op->GetDataInstance();
	if (!data)
		return false;

	BaseDocument* doc = GetActiveDocument();
	BaseTime time = doc->GetMaxTime();
	Int32 fps = doc->GetFps();
	Int32 mxFrame = time.GetFrame(fps);

	data->SetInt32(AN_CLONER_FROM, 0);
	data->SetInt32(AN_CLONER_TO, mxFrame);
	data->SetInt32(AN_CLONER_SEED, 1234);
	data->SetInt32(AN_CLONER_SPAN, 30);
	data->SetFloat(AN_CLONER_MULTIPLIER, 1);

	data->SetFloat(ID_MG_BASEEFFECTOR_MINSTRENGTH, -1.0);
	data->SetBool(ID_MG_BASEEFFECTOR_POSITION_ACTIVE, true);
	data->SetVector(ID_MG_BASEEFFECTOR_POSITION, Vector(50.0));
	data->SetFloat(1100, 0.01);
	data->SetInt32(SINGLE_OBJ_ROTATION_ORDER, SINGLE_OBJ_ROTATION_ORDER_HPB);

	InitSpline(*data, AN_CLONER_SPLINE);
	splineAnimate = (SplineData*)data->GetData(AN_CLONER_SPLINE).GetCustomDataType(CUSTOMDATATYPE_SPLINE);

	return true;
}


//Bool EchoMotion::Message(GeListNode* node, Int32 type, void* t_data)
//{
//	//switch (type) 
//	//{
//	//	case MSG_DESCRIPTION_COMMAND :
//	//	{
//	//		BaseContainer* bc = ((BaseObject*)node)->GetDataInstance();
//	//		DescriptionCommand* dc = (DescriptionCommand*)t_data;
//	//		const Int32 id = dc->_descId[0].id;
//
//	//		switch (id)
//	//		{
//	//				case AN_CLONER_UPDATE:// Update Button:
//	//				{
//
//	//					break;
//	//				}
//	//		}
//	//		break;
//	//	}
//	//	case MSG_DESCRIPTION_POSTSETPARAMETER:
//	//	{
//	//		BaseContainer* nodeCon = ((BaseObject*)node)->GetDataInstance();
//	//		DescriptionPostSetValue* dc = (DescriptionPostSetValue*)t_data;
//
//	//		const DescID* desc = dc->descid;
//	//		
//	//		
//	//		// Handling CheckBoxes dependencies:
//	//		DescID paridSingleObj = DescLevel(AN_CLONER_TOGGLE_SINGLE_OBJ, DTYPE_BOOL, 0);
//	//		if (paridSingleObj.IsPartOf(*desc, nullptr)) 
//	//		{
//	//			if (nodeCon->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ)) 
//	//			{
//	//				nodeCon->SetBool(AN_CLONER_SPLINE_TOGGLE, false);
//	//			}
//	//		}
//	//		DescID paridSpl = DescLevel(AN_CLONER_SPLINE_TOGGLE, DTYPE_BOOL, 0);
//	//		if (paridSpl.IsPartOf(*desc, nullptr)) 
//	//		{
//	//			if (nodeCon->GetBool(AN_CLONER_SPLINE_TOGGLE)) 
//	//			{
//	//				nodeCon->SetBool(AN_CLONER_TOGGLE_SINGLE_OBJ, false);
//	//			}
//	//		}
//	//		// Handling relational dependencies for the range between "From" and "To" so it shouldn't be less than range = 5
//	//		Int32 range = 5;// the min range allowed
//	//		Int32 From = nodeCon->GetFloat(AN_CLONER_FROM);
//	//		Int32 To = nodeCon->GetFloat(AN_CLONER_TO);
//	//		if (abs(To - From )< range) 
//	//		{
//	//			DescID paridFrom = DescLevel(AN_CLONER_FROM, DTYPE_LONG, 0);
//	//			DescID paridTo = DescLevel(AN_CLONER_TO, DTYPE_LONG, 0);
//	//			//		if From parameter is being called:
//	//			if (paridFrom.IsPartOf(*desc, nullptr)) 
//	//			{
//	//				if (From <= To) 
//	//				{
//	//					nodeCon->SetFloat(AN_CLONER_FROM, To - range);
//	//				}
//	//				else 
//	//				{
//	//					nodeCon->SetFloat(AN_CLONER_FROM, To + range);
//	//				}
//	//			}
//	//			//		elif To parameter is being called:
//	//			else if (paridTo.IsPartOf(*desc, nullptr)) 
//	//			{
//	//				if (To <= From) 
//	//				{
//	//					nodeCon->SetFloat(AN_CLONER_TO, From - range);
//	//				}
//	//				else 
//	//				{
//	//					nodeCon->SetFloat(AN_CLONER_FROM, From + range);
//	//				}
//	//			}
//	//		}
//	//		break;
//	//	}
//	// }
//	return true;
//}

Bool EchoMotion::GetDDescription(GeListNode* node, Description* description, DESCFLAGS_DESC& flags)
{
	if (!description->LoadDescription(node->GetType()))
		return false;

	Int32 mxFrame = GetActiveDocument()->GetMaxTime().GetFrame(GetActiveDocument()->GetFps());

	// important to check for speedup c4d!
	const DescID* singleid = description->GetSingleDescID();

	DescID paridTo = DescLevel(AN_CLONER_TO, DTYPE_LONG, 0);

	if (!singleid || paridTo.IsPartOf(*singleid, nullptr))	// important to check for speedup c4d!
	{
		BaseContainer* descBc = description->GetParameterI(paridTo, nullptr);
		descBc->SetInt32(DESC_MAX, mxFrame);
	}

	DescID paridFrom = DescLevel(AN_CLONER_FROM, DTYPE_LONG, 0);
	if (!singleid || paridFrom.IsPartOf(*singleid, nullptr))	// important to check for speedup c4d!
	{
		BaseContainer* descBc = description->GetParameterI(paridFrom, nullptr);
		descBc->SetInt32(DESC_MAX, mxFrame);
	}

	flags |= DESCFLAGS_DESC::LOADED;
	return true;
}


Bool EchoMotion::GetDEnabling(GeListNode* node, const DescID& id, const GeData& t_data, DESCFLAGS_ENABLE flags, const BaseContainer* itemdesc) {
	BaseContainer* data = ((BaseObject*)node)->GetDataInstance();
	switch (id[0].id)
	{
		case AN_CLONER_SPLINE:
			return (data->GetBool(AN_CLONER_SPLINE_TOGGLE));
		case AN_CLONER_SINGLE_OBJ:
			return (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case SINGLE_OBJ_ROTATION_ORDER:
			return (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_ID:
			return !(data->GetBool(AN_CLONER_SPLINE_TOGGLE)) && !(data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_SEED:
			return !(data->GetBool(AN_CLONER_SPLINE_TOGGLE)) && !(data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_FROM:
			return (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_TO:
			return (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_SPAN:
			return (data->GetBool(AN_CLONER_SPLINE_TOGGLE)) || (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
		case AN_CLONER_MULTIPLIER:
			return (data->GetBool(AN_CLONER_SPLINE_TOGGLE)) || (data->GetBool(AN_CLONER_TOGGLE_SINGLE_OBJ));
	}	
	
	return true;
}


void EchoMotion::InitPoints(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, MoData* md, BaseThread* thread)
{
	Int32 mdcount = (Int32)md->GetCount();
	BaseContainer* bc = op->GetDataInstance();
	if (!bc)
		return;
	// Update Arrays count if generator's count changed
	Int32 cntDirty = md->GetDirty(MDDIRTY::COUNT);
	if (lastCountDirty != cntDirty)
	{
		lastCountDirty = cntDirty;
		if (trigger_frame.Resize(mdcount) == maxon::FAILED) 
			return;
		if (trigger_status.Resize(mdcount) == maxon::FAILED) 
			return;
		if (falloff_history.Resize(mdcount) == maxon::FAILED) 
			return;
		if (random_pick.Resize(mdcount) == maxon::FAILED) 
			return;
		//ApplicationOutput(String::IntToString(trigger_frame.GetCount()));
	}
}

void EchoMotion::ModifyPoints(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, MoData* md, BaseThread* thread)
{
			// Grabbing the field
	C4D_Falloff* falloff = GetFalloff();
	FieldOutput* fieldSample = nullptr;
	if (!falloff)
	{
		fieldSample = CalcFields(doc, gen, md, FIELDSAMPLE_FLAG::VALUE);
		if (!fieldSample || !fieldSample->_value.IsPopulated())
			return;
	}
	BaseContainer* bc = op->GetDataInstance();
	// Time:
	Int32 cf = (doc->GetTime()).GetFrame(doc->GetFps());
	// MoData :
	Int32 mdcount = (Int32)md->GetCount();
	Int32	 i = 0;
	Float	 fall = 0.0;
	Vector off = Vector(0.0);
	MDArray<Int32>	flag_array = md->GetLongArray(MODATA_FLAGS);
	MDArray<Matrix> mat_array = md->GetMatrixArray(MODATA_MATRIX);
	MDArray<Float>	weight_array = md->GetRealArray(MODATA_WEIGHT);
	// Main Loop over the clones :
	for (i = 0; i < mdcount; i++)
	{
		// Multiply into global space
		off = mat_array[i].off;
		Matrix genmg = gen->GetMg();
		off = genmg * off;
		// Sample the falloff
		if (falloff)
		{
			falloff->Sample(off, &fall, true, weight_array[i]);
		}
		else
		{
			fall = fieldSample->_value[i];
		}
		// Setting trigger event:
		if (((fieldSample->_value[i]) != falloff_history[i]) && !trigger_status[i])
		{
			if (fieldSample->_value[i] != 0.0) // if the field has entered it
			{
				trigger_status[i] = true;
				trigger_frame[i] = cf;
			}
		}
		Float x = mat_array[i].off.x , y = mat_array[i].off.y , z = mat_array[i].off.z;
		// if Spline Mode enabled
		Int32 span = bc->GetInt32(AN_CLONER_SPAN);
		Float multiplier = bc->GetFloat(AN_CLONER_MULTIPLIER);
		// if clone in Trigger mode
		if (trigger_status[i])
		{
			if (trigger_frame[i] <= cf && cf <= trigger_frame[i] + span)
			{
				Float proportionalFrame = (cf - trigger_frame[i]);
				Float Division = proportionalFrame / span;
				if (splineAnimate)
				{
					mat_array[i].off = Vector(x, y + splineAnimate->GetPoint(Division).y * multiplier * 100, z);
				}
			}
			else
			{
				trigger_status[i] = false;
			}
		}
		
		// Mode-Independent
		if (cf == doc->GetMaxTime().GetFrame(doc->GetFps()))
			trigger_status[i] = false;
		// Updating history for comparison with life falloff value	
		falloff_history[i] = fieldSample->_value[i];
	}
}
//
//void EchoMotion::CalcPointValue(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, Int32 index, MoData* md, const Vector& globalpos, Float fall_weight)
//{
//
//}
//
//Vector EchoMotion::CalcPointColor(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, Int32 index, MoData* md, const Vector& globalpos, Float fall_weight)
//{
//	return Vector(Noise(globalpos), Noise(globalpos + Vector(20.0)), Noise(globalpos - Vector(20.0)));
//}

