#ifndef ECHOMOTION_H_
#define ECHOMOTION_H_

// #include <vector>
#include "c4d.h"
#include "c4d_symbols.h"
#include "c4d_baseeffectordata.h"
#include "c4d_falloffplugin.h"
#include "oeecho.h"


#define ID_ECHOMOTIOM 1061990

class EchoMotion : public EffectorData
{
public:
	Int32  lastCountDirty;// Count dirty
	Int32  lastSingleDirty;// Count dirty
	maxon::BaseArray<Int32> trigger_frame;
	maxon::BaseArray<Bool> trigger_status;
	maxon::BaseArray<Float> falloff_history;
	maxon::BaseArray<Int32> random_pick;

	// Splines for easy access and to avoid allocating splines when needing to use them:
	SplineData* splineAnimate;


	EchoMotion();
	virtual ~EchoMotion();
	virtual Bool InitEffector(GeListNode* node);
	//virtual Bool Message(GeListNode* node, Int32 type, void* t_data);
	virtual Bool GetDDescription(GeListNode* node, Description* description, DESCFLAGS_DESC& flags);
	virtual Bool GetDEnabling(GeListNode* node, const DescID& id, const GeData& t_data, DESCFLAGS_ENABLE flags, const BaseContainer* itemdesc);
	

	virtual void InitPoints(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, MoData* md, BaseThread* thread);
	
	void ModifyPoints(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, MoData* md, BaseThread* thread);
	/*virtual void CalcPointValue(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, Int32 index, MoData* md, const Vector& globalpos, Float fall_weight);
	virtual Vector CalcPointColor(BaseObject* op, BaseObject* gen, BaseDocument* doc, EffectorDataStruct* data, Int32 index, MoData* md, const Vector& globalpos, Float fall_weight);*/


	static NodeData* Alloc() { return NewObjClear(EchoMotion); };
};


Bool RegisterEchoMotion() {
	return RegisterEffectorPlugin(ID_ECHOMOTIOM, GeLoadString(IDS_ECHOMOTION), OBJECT_CALL_ADDEXECUTION, EchoMotion::Alloc, "oeecho"_s, AutoBitmap("EchoMotion.tif"_s), 0);
}






static void InitSpline(BaseContainer& data, Int32 id, Int32 Yrange = 1, Float step = 0.1)
{
	GeData d(CUSTOMDATATYPE_SPLINE, DEFAULTVALUE);

	SplineData* p = (SplineData*)d.GetCustomDataType(CUSTOMDATATYPE_SPLINE);
	if (p)
	{
		p->SetRange(0, 1, 0.1, -Yrange, Yrange, 0.1);

		p->MakeLinearSplineBezier(2);  // create spline
		p->Flip();

		Int32 count = p->GetKnotCount();
		if (count == 2)
		{
			CustomSplineKnot* knot0 = p->GetKnot(0);
			if (knot0)
			{
				knot0->vPos = Vector(0, 1.0, 0);
				knot0->vTangentLeft = Vector(-0.3, 0, 0);
				knot0->vTangentRight = Vector(0.3, 0, 0);
			}

			CustomSplineKnot* knot2 = p->GetKnot(1);
			if (knot2)
			{
				knot2->vPos = Vector(1, 0, 0);
				knot2->vTangentLeft = Vector(-0.3, 0, 0);
				knot2->vTangentRight = Vector(0.3, 0, 0);
			}
		}
	}
	data.SetData(id, d);
}





#endif // ECHOMOTION_H_
